/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.annotation;

import java.lang.annotation.Documented;
import java.lang.annotation.ElementType;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;

/**
 * This annotation modifies the configuration associated to the 
 * annotated event method. 
 * @author Ignacio Coloma
 */
@Retention(RetentionPolicy.RUNTIME)
@Target({ElementType.METHOD})
@Documented
public @interface Event {

	/** 
	 * True if this is the default event, default to false 
	 * Default evvents are triggered when the 'event' value is missing
	 */
	boolean defaultEvent() default false;
	
	/**
	 * This method will be called before action.setContext().
	 * This is the first listener to get called, and is typically used 
	 * for action postProcessing such as injecting IoC beans and such.
	 */
	String[] afterCreate() default "";
	
	/**
	 * This method will be called before population (injecting the request 
	 * values into action properties),
	 * except any property annotated with {@link EagerProperty}.
	 * This handler is the intended place to retrieve data from any database,
	 * using the configured eager properties as primary keys.
	 */
	String[] beforePopulate() default "";
	
	/**
	 * This method will be called after population and before the validation
	 * of the action properties.
	 */
	String[] beforeValidate() default "";
	
	/**
	 * This method will be called if validation/conversion errors have happened.
	 */
	String[] onError() default "";
	
	/**
	 * This method will be called after validation and before calling the 
	 * action event method
	 */
	String[] beforeExecute() default "";
	
	/**
	 * This method will be called after the request has been processed,
	 * even if there is any exception.
	 * Use this for any resource cleanup.
	 */
	String[] doFinally() default "";
	
}
