/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.annotation;

import java.lang.annotation.Documented;
import java.lang.annotation.ElementType;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;

/**
 * Marks a event argument to receive all parameters of a request. For example, given a class:
 * 
 * <pre>
 * public class Model {
 * 
 * 	private Integer id;
 * 
 *  private String name;
 * 
 * }
 * </pre>
 * 
 * And Event:
 * 
 * <pre>
 * public Resolution sync(@RequestParameters Model model) {
 * }
 * </pre>
 * 
 * Any id or name parameters will be directly injected into model. Note that if 
 * this annotation is used, all parameters from the request must be injected into 
 * the annotated object, and if it is not possible the request will fail.
 * 
 * This annotation is specially useful when injecting application/json
 * requests (such as the case of backbone.js applications) 
 * 
 * @author Ignacio Coloma
 */
@Retention(RetentionPolicy.RUNTIME)
@Target(ElementType.PARAMETER)
@Documented
public @interface RequestParameters {

}
