package org.loom.annotation.processor;

import java.util.List;
import java.util.Set;
import java.util.SortedSet;
import java.util.TreeSet;

import javax.inject.Singleton;

import org.loom.action.Action;
import org.loom.log.Log;
import org.loom.mapping.ActionMapping;
import org.loom.mapping.Event;

import com.google.common.collect.Lists;

@Singleton
public class DefaultAnnotationProcessorRepository implements AnnotationProcessorRepository {

	/** processes annotations on methods and properties. By default it has the native loom annotations */
	private SortedSet<AnnotationProcessor> annotationProcessors= new TreeSet<AnnotationProcessor>();

	private static Log log = Log.getLog(AnnotationProcessorRepository.class);

	@Override
	public void process(ActionMapping mapping) {
		
		// process attribute annotations
		Class<? extends Action> actionClass = mapping.getActionClass();
		log.debug("Start processing annotations for ", actionClass.getName());
		VisitedClasses.initialize();
		VisitedClasses.add(actionClass);
		long t1 = System.currentTimeMillis();
		for (AnnotationProcessor processor : annotationProcessors) {
			// workaround: ConcurrentModificationException will happen if any processor modifies the list of events
			// TODO: there must be a better solution for this. Modifying the list of events doesn't look right
			List<Event> events = Lists.newArrayList(mapping.getEvents().values()); 
			for (Event event: events) {
				processor.process(event);
			}
		}
		long t2 = System.currentTimeMillis();
		log.debug("Processed ", mapping.getEvents().size(), " events in ", (t2 - t1), " milliseconds");
		
		// the list of reloading classes will be set to the list of visited classes
		mapping.setReloadingClasses(VisitedClasses.get());
	}

	public void addAnnotationProcessor(AnnotationProcessor processor) {
		log.debug("Adding annotation processor: ", processor.getClass().getName());
		annotationProcessors.add(processor);
	}

	public Set<AnnotationProcessor> getAnnotationProcessors() {
		return annotationProcessors;
	}
	
}
