/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.annotation.processor;

import java.math.BigDecimal;

import org.loom.annotation.validation.NumberValidation;
import org.loom.annotation.validation.NumberValidations;
import org.loom.mapping.Event;
import org.loom.validator.NumberValidator;


/**
 * Processes {@link NumberValidation} annotations
 * @author icoloma
 *
 */
public class NumberValidationAnnotationProcessor extends AbstractPropertyAnnotationProcessor<NumberValidation, NumberValidations> {

	public NumberValidationAnnotationProcessor() {
		super(NumberValidation.class, NumberValidations.class);
	}

	@Override
	protected void process(Event event, NumberValidation annotation, String propertyPath) {
		NumberValidator validator = new NumberValidator();
		validator.setBindMessages(annotation.bindMessages());
		validator.setPropertyPath(propertyPath);
		if (annotation.message().length() > 0) {
			validator.setOverridenMessageKey(annotation.message());
		}
		if (annotation.minValue().length() > 0) {
			validator.setMinValue(new BigDecimal(annotation.minValue()));
		}
		if (annotation.maxValue().length() > 0) {
			validator.setMaxValue(new BigDecimal(annotation.maxValue()));
		}
		validator.setExcludeMin(annotation.excludeMin());
		validator.setExcludeMax(annotation.excludeMax());
		if (annotation.scale() != -1) {
			validator.setScale(annotation.scale() );
		}
		if (annotation.precision() != -1) {
			validator.setPrecision(annotation.precision() );
		}
		event.addValidator(validator);
	}

}
