/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.annotation.processor;

import java.util.regex.Pattern;

import org.loom.annotation.validation.StringValidation;
import org.loom.annotation.validation.StringValidations;
import org.loom.mapping.Event;
import org.loom.validator.StringValidator;


/**
 * Processes {@link StringValidation} annotations
 * @author icoloma
 *
 */
public class StringValidationAnnotationProcessor extends AbstractPropertyAnnotationProcessor<StringValidation, StringValidations> {

	public StringValidationAnnotationProcessor() {
		super(StringValidation.class, StringValidations.class);
	}
	
	@Override
	protected void process(Event event, StringValidation annotation, String propertyPath) {
		StringValidator validator = new StringValidator();
		validator.setBindMessages(annotation.bindMessages());
		validator.setPropertyPath(propertyPath);
		if (annotation.message().length() > 0) {
			validator.setOverridenMessageKey(annotation.message());
		}
		if (annotation.maxLength() != -1) {
			validator.setMaxLength(annotation.maxLength());
		}
		if (annotation.minLength() != -1) {
			validator.setMinLength(annotation.minLength());
		}
		if (annotation.mask().length() > 0) {
			Pattern p = Pattern.compile(annotation.mask(), annotation.maskCaseInsensitive()? Pattern.CASE_INSENSITIVE : 0);
			validator.setMaskPattern(p);
		}
		event.addValidator(validator);
	}

	
}
