package org.loom.annotation.validation;

import java.lang.annotation.Documented;
import java.lang.annotation.ElementType;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;

import org.loom.servlet.params.FileParameter;

/**
 * Validates a {@link FileParameter} that is expected to contain only images
 * @author Rafael Serrano
 */
@Retention(RetentionPolicy.RUNTIME)
@Target({ ElementType.METHOD })
@Documented
public @interface ImageValidation {
	
	/** name of the expected parameter */
	String parameterName();
	
    /** if provided, only allows the specified image formats (gif, jpg, jpeg, bmp, png, tiff) */
    String[] formats() default {};
    
    /** the minimum width of the image, defaults to -1 */
    int minWidth() default -1;
    
    /** the maximum width of the image, defaults to -1 */
    int maxWidth() default -1;

    /** the minimum height of the image, defaults to -1 */
    int minHeight() default -1;
    
    /** the maximum height of the image, defaults to -1 */
    int maxHeight() default -1;
    
    /** the maximum file size in bytes, defaults to -1 */
    int maxFileSize() default -1;
    
}
