/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.binding;

import org.loom.binding.factories.ImplementationFactory;
import org.loom.converter.Converter;
import org.loom.exception.ConfigException;

/**
 * The item metadata associated to a multiple field (array, list, set, map).
 * If this multiple field can be an intermediate node of a property path, it should provide 
 * a valid ImplementationFactory. If it is an end node of a property path, it should provide
 * a valid Converter.
 * @author icoloma
 */
public class ItemMetadata {

	/** the item implementation class */
	private Class<?> itemClass;

	/** the item Converter */
	private Converter converter;

	/** the item implementation factory */
	private ImplementationFactory<?> implementationFactory;

	/**
	 * @return true if neither converter or itemClass has been specified
	 */
	public boolean isEmpty() {
		return converter == null && itemClass == null;
	}

	public Converter getConverter() {
		return converter;
	}

	public void setConverter(Converter converter) {
		this.converter = converter;
	}

	public Class<?> getItemClass() {
		return itemClass;
	}

	public void setItemClass(Class<?> itemImplementationClass) {
		this.itemClass = itemImplementationClass;
	}

	public ImplementationFactory<?> getImplementationFactory() {
		return implementationFactory;
	}

	public void setImplementationFactory(ImplementationFactory<?> implementationFactory) {
		this.implementationFactory = implementationFactory;
	}

	public Object newItem() {
		if (implementationFactory == null)
			throw new ConfigException("Cannot create node. No implementationFactory has been set");
		return implementationFactory.newInstance();
	}
	
}
