/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.config;

import javax.inject.Inject;
import javax.inject.Singleton;

import org.codehaus.jackson.map.ObjectMapper;
import org.loom.binding.PropertyWrapperFactory;
import org.loom.exception.ExceptionHandler;
import org.loom.i18n.MessagesRepositoryFactory;
import org.loom.log.Log;
import org.loom.mapping.ActionMappingRepository;
import org.loom.resolution.ResolutionFactory;
import org.loom.resources.WebResourceBundleRepository;
import org.loom.servlet.LoomServletRequestFactory;
import org.loom.tags.ExpressionLanguageProxy;
import org.loom.tags.decorator.TagDecoratorRepository;
import org.loom.util.Lookup;

/**
 * Loom configuration. The configuration class must be unique per servlet, but
 * could be more than one per web container
 * 
 * @deprecated use {@link Lookup} or plain injection instead.
 * @author icoloma
 */
@Singleton
public class Config {

	/** repository with all configured ActionMapping instances */
	@Inject
	private ActionMappingRepository actionMappingRepository;

	@Inject
	private LoomServletRequestFactory loomServletRequestFactory;

	/** TagDecorator instances repository */
	@Inject
	private TagDecoratorRepository tagDecoratorRepository;

	/** the repository of {@link WebResource} to use */
	@Inject
	private WebResourceBundleRepository webResourceBundleRepository;

	/** the class that executes EL expressions inside loom */
	@Inject
	private ExpressionLanguageProxy elProxy;
	
	@Inject
	private PropertyWrapperFactory propertyWrapperFactory;
	
	/** the {@link ResolutionFactory} to be used for creating Redirect and Forward resolutions */
	@Inject
	private ResolutionFactory resolutionFactory;
	
	/** the {@link ObjectMapper} used to serialize/deserialize JSON objects */
	@Inject
	private ObjectMapper objectMapper;

	@Inject
	private ExceptionHandler exceptionHandler;
	
	//private ServletContext servletContext;
	
	private static Log log = Log.getLog(Config.class);

	/** the singleton instance */
	//private static Config instance;

	public ActionMappingRepository getActionMappingRepository() {
		return actionMappingRepository;
	}
/*
	public static final Config getInstance() {
		return instance;
	}

	public static final void setInstance(Config config) {
		Config.instance = config;
	}
*/
	public TagDecoratorRepository getTagDecoratorRepository() {
		return tagDecoratorRepository;
	}

	public void setActionMappingRepository(ActionMappingRepository actionMappingRepository) {
		this.actionMappingRepository = actionMappingRepository;
	}

	public void setTagDecoratorRepository(TagDecoratorRepository tagDecoratorRepository) {
		this.tagDecoratorRepository = tagDecoratorRepository;
	}

	public ExpressionLanguageProxy getElProxy() {
		return elProxy;
	}

	public ResolutionFactory getResolutionFactory() {
		return resolutionFactory;
	}

	public void setResolutionFactory(ResolutionFactory resolutionFactory) {
		this.resolutionFactory = resolutionFactory;
	}

	public ObjectMapper getObjectMapper() {
		return objectMapper;
	}

	public void setObjectMapper(ObjectMapper objectMapper) {
		this.objectMapper = objectMapper;
	}

	public PropertyWrapperFactory getPropertyWrapperFactory() {
		return propertyWrapperFactory;
	}
	
	public void setPropertyWrapperFactory(PropertyWrapperFactory propertyWrapperFactory) {
		this.propertyWrapperFactory = propertyWrapperFactory;
	}

	public ExceptionHandler getExceptionHandler() {
		return exceptionHandler;
	}

	public void setExceptionHandler(ExceptionHandler exceptionTranslator) {
		this.exceptionHandler = exceptionTranslator;
	}

	public void setElProxy(ExpressionLanguageProxy elProxy) {
		this.elProxy = elProxy;
	}

	public WebResourceBundleRepository getWebResourceBundleRepository() {
		return webResourceBundleRepository;
	}

	public void setWebResourceBundleRepository(
			WebResourceBundleRepository webResourceBundleRepository) {
		this.webResourceBundleRepository = webResourceBundleRepository;
	}

	public LoomServletRequestFactory getLoomServletRequestFactory() {
		return loomServletRequestFactory;
	}

	public void setLoomServletRequestFactory(
			LoomServletRequestFactory loomServletRequestFactory) {
		this.loomServletRequestFactory = loomServletRequestFactory;
	}

	/**
	 * @deprecated Use getLoomServletRequestFactory().getMessagesRepositoryFactory() instead
	 */
	public MessagesRepositoryFactory getMessagesRepositoryFactory() {
		return loomServletRequestFactory.getMessagesRepositoryFactory();
	}

	/**
	 * @deprecated Use getLoomServletRequestFactory().getLocaleResolver() instead
	 */
	public LocaleResolver getLocaleResolver() {
		return loomServletRequestFactory.getLocaleResolver();
	}

}
