package org.loom.config;

import java.util.Properties;

import org.loom.tags.AbstractHtmlTag;
import org.loom.util.PropertiesUtils;

/**
 * Contains the contents of the loom-config.properties  file in the classpath
 * @author icoloma
 *
 */
public class ConfigProperties {

	private static ConfigProperties instance = new ConfigProperties();

	/**
	 * the configuration properties, that are read by default from
	 * classpath:resources/loom-messages
	 */
	private Properties properties;
	
	public ConfigProperties() {
		this(PropertiesUtils.loadFromClasspath("META-INF/loom-config.properties"));
	}
	
	public ConfigProperties(Properties properties) {
		this.properties = properties;
	}

	public static ConfigProperties getInstance() {
		return instance;
	}
	
	public static void setInstance(ConfigProperties instance) {
		ConfigProperties.instance = instance;
	}

	public void setProperty(String name, String value) {
		this.properties.setProperty(name, value);
	}

	/**
	 * @return a configured property, as Integer
	 */
	public Integer getPropertyAsInteger(String propertyKey, Integer defaultValue) {
		String result = properties.getProperty(propertyKey);
		return result == null ? defaultValue : Integer.valueOf(result);
	}

	/**
	 * @return a configured property, as boolean
	 */
	public boolean getPropertyAsBoolean(String propertyKey, boolean defaultValue) {
		String s = properties.getProperty(propertyKey);
		return s == null ? defaultValue : Boolean.valueOf(s);
	}

	/**
	 * @return a configured property, as String
	 */
	public String getProperty(String propertyKey, String defaultValue) {
		String s = properties.getProperty(propertyKey);
		return s == null ? defaultValue : s;
	}

	/**
	 * @return the default CSS value for the specified tag, defaultCSS if there
	 *         is none
	 */
	public String getDefaultCss(Class<? extends AbstractHtmlTag> clazz, String defaultCSS) {
		return getProperty("css." + clazz.getSimpleName(), defaultCSS);
	}
	
}
