/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.config;

import java.util.Locale;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * Return the locale to use while handling a request object.
 * @author icoloma
 */
public interface LocaleResolver {

	/**
	 * Extract the locale from the user request, null if it no serialized locale could be found.
	 * If null, the locale from the request instance will be used.
	 * @param request the current user request
	 * @return the locale to  apply for this request
	 */
	public Locale getLocale(HttpServletRequest request);
	
	/**
	 * Change the locale on the user request
	 * @param response the response to set the locale into
	 * @param locale the new Locale to apply
	 * @throws UnsupportedOperationException if this LocaleResolver cannot set a new Locale value
	 */
	public void setLocale(HttpServletResponse response, Locale locale);
	
}
