/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.config;

import java.util.Locale;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.inject.Singleton;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.loom.util.LocaleUtils;

/**
 * Locale managed as a hostname prefix, eg: es.foo.com.
 * @author icoloma
 */
@Singleton
public class PrefixLocaleResolver implements LocaleResolver {
	
	/** the regex that will be used to extract the locale from the URL, defaults to only language code */
	private Pattern localePattern = Pattern.compile("^([a-z][a-z])\\.");
	
	public Locale getLocale(HttpServletRequest request) {
		Matcher matcher = localePattern.matcher(request.getServerName());
		if (matcher.find()) {
			return LocaleUtils.toLocale(matcher.group(1));
		}
		return null;
	}

	public void setLocale(HttpServletResponse response, Locale locale) {
		// this resolver cannot set the new locale 
		throw new UnsupportedOperationException();
	}

	public void setLocalePattern(Pattern localePattern) {
		this.localePattern = localePattern;
	}

}
