/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.converter;

import java.text.Format;
import java.text.ParsePosition;

import org.loom.i18n.Messages;
import org.loom.i18n.MessagesRepository;


/**
 * Converter that is based in a {@link Format}
 * @author icoloma
 */
public abstract class AbstractFormatConverter extends AbstractStringConverter {

	/** the key that will be used to search for the format string in the MessagesRepository*/
	private String formatKey;
	
	/** the key for the error message in the MessagesRepository*/
	private String errorKey;

	public AbstractFormatConverter(Class<?> convertedClass, String formatKey, String errorKey) {
		super(convertedClass);
		this.formatKey = formatKey;
		this.errorKey = errorKey;
	}
	
	public AbstractFormatConverter(Class<?> convertedClass, String formatKey) {
		this(convertedClass, formatKey, "loom.conversion.numberFailed");
	}

	@Override
	public Object getAsObjectImpl(String paramName, String paramValue, Messages messages, MessagesRepository repository) {
		ParsePosition pos = new ParsePosition(0);
		Object result = parse(paramValue, pos, repository);
		if (pos.getIndex() != paramValue.length()) {
			addErrorMessage(messages, paramName, paramValue, errorKey);
			return null;
		}
		return result;
	}

	protected Object parse(String text, ParsePosition pos, MessagesRepository repository) {
		return createFormat(repository).parseObject(text, pos);
	}

	@Override
	public String getAsTextImpl(Object value, MessagesRepository repository) {
		return createFormat(repository).format(value);
	}

	/** Creates a new format instance. Format instances may be different per locale and will be cached in the MessagesRepository */
	protected abstract Format createFormat(MessagesRepository messagesRepository);

	public String getFormatKey() {
		return formatKey;
	}

}
