/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.converter.date;

import java.text.FieldPosition;
import java.text.Format;
import java.text.ParsePosition;

import org.joda.time.DateTime;
import org.joda.time.format.DateTimeFormatter;

/**
 * Bridge between java.text.Format and org.joda.time.format.DateTimeFormatter.
 * @author icoloma
 */
public abstract class AbstractDateFormatAdapter<T> extends Format {
	
	private DateTimeFormatter formatter;

	@Override
	@SuppressWarnings("unchecked")
	public final StringBuffer format(Object obj, StringBuffer toAppendTo, FieldPosition pos) {
		formatImpl((T)obj, toAppendTo);
		return toAppendTo;
	}

	protected abstract void formatImpl(T obj, StringBuffer toAppendTo);
	
	@Override
	public final Object parseObject(String source, ParsePosition pos) {
		try {
			DateTime dateTime = formatter.parseDateTime(source);
			pos.setIndex(source.length());
			return toResultClass(dateTime);
		} catch (RuntimeException e) {
			// error indicated by the unaltered ParsePosition
			return null;
		}
	}

	protected abstract T toResultClass(DateTime dateTime);

	public DateTimeFormatter getFormatter() {
		return formatter;
	}

	public void setFormatter(DateTimeFormatter formatter) {
		this.formatter = formatter;
	}
	
}
