/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.converter.date;

import java.text.Format;

import org.joda.time.format.DateTimeFormat;
import org.joda.time.format.DateTimeFormatter;
import org.loom.converter.AbstractFormatConverter;
import org.loom.i18n.MessagesRepository;
import org.loom.servlet.names.CssClassNames;
import org.loom.tags.FormInputTag;
import org.loom.tags.core.AbstractTextFieldTag;
import org.loom.util.ClassUtils;


/**
 * Converter for java.util.Date
 * @author icoloma
 */
public final class DateConverter extends AbstractFormatConverter {

	/** the date format key */
	public static final String DATE_FORMAT_KEY = "loom.format.date";
	
	/** the date + time format key */
	public static final String DATE_TIME_FORMAT_KEY = "loom.format.dateTime";
	
	/** time without date format key */
	public static final String TIME_FORMAT_KEY = "loom.format.time";
	
	/** the type of class that will be returned */
	private Class<? extends AbstractDateFormatAdapter<?>> formatAdapterClass;

	public DateConverter(Class<?> dateClass, String dateFormatKey, Class<? extends AbstractDateFormatAdapter<?>> formatAdapterClass) {
		super(dateClass, dateFormatKey, "loom.conversion.dateFailed");
		this.formatAdapterClass = formatAdapterClass;
		String formatKey = getFormatKey();
		if (DATE_FORMAT_KEY.equals(formatKey)) {
			setCssClass(CssClassNames.DATE);
		} else if (DATE_TIME_FORMAT_KEY.equals(formatKey)) {
			setCssClass(CssClassNames.DATE_TIME);
		} else if (TIME_FORMAT_KEY.equals(formatKey)) {
			setCssClass(CssClassNames.TIME);
		}
	}
	
	@Override
	protected Format createFormat(MessagesRepository messagesRepository) {
		AbstractDateFormatAdapter<?> wrapper = ClassUtils.newInstance(formatAdapterClass);
		String pattern = messagesRepository.getRequiredString(getFormatKey());
		DateTimeFormatter formatter = DateTimeFormat.forPattern(pattern).withLocale(messagesRepository.getLocale());
		wrapper.setFormatter(formatter);
		return wrapper;
	}
	
	@Override
	public void decorate(FormInputTag t) {
		super.decorate(t);
		if (t instanceof AbstractTextFieldTag) {
			AbstractTextFieldTag tag = (AbstractTextFieldTag)t;
			// Admittedly could be wrong, but for the usual case it works: the maximum input string
			// is the same length as the mask string (dd/MM/yyy and all that).
			// If that's not your case, you are acting Weird and God will punish you.
			if (tag.getMaxlength() == null) {
				tag.setMaxlength(tag.getMessagesRepository().getRequiredString(getFormatKey()).length());
			}
		}
	}
		
}

