/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.converter.date;

import java.sql.Time;
import java.sql.Timestamp;

import javax.persistence.Temporal;

import org.loom.binding.AnnotationContainer;
import org.loom.converter.AnnotationDrivenConverterFactory;
import org.loom.converter.Converter;

/**
 * Converter factory for JPA annotated date properties
 * @author icoloma
 */
public class TemporalConverterFactory implements AnnotationDrivenConverterFactory<Temporal> {

	private Converter date = new DateConverter(java.util.Date.class, DateConverter.DATE_FORMAT_KEY, UtilDateAdapterFormat.class);
	private Converter dateWithTime = new DateConverter(java.util.Date.class, DateConverter.DATE_TIME_FORMAT_KEY, UtilDateAdapterFormat.class);
	private Converter sqlDate = new DateConverter(java.sql.Date.class, DateConverter.DATE_FORMAT_KEY, SqlDateAdapterFormat.class);
	private Converter timestamp = new DateConverter(Timestamp.class, DateConverter.DATE_TIME_FORMAT_KEY, TimestampAdapterFormat.class);
	private Converter time = new DateConverter(Time.class, DateConverter.TIME_FORMAT_KEY, TimeAdapterFormat.class);
	
	public Converter getConverter(AnnotationContainer container, Temporal annotation) {
		Class<?> clazz = container.getPropertyClass();
		switch (annotation.value()) {
		case DATE:
			return java.sql.Date.class == clazz? sqlDate : date;
		case TIMESTAMP:
			return java.sql.Timestamp.class == clazz? timestamp : dateWithTime;
		case TIME:
			return time;
		default:
			throw new IllegalArgumentException("Unsupported TemporalType value: " + annotation.value());
		}
	}

	public Class<Temporal> getAnnotationClass() {
		return Temporal.class;
	}
}
