/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.filter;

import java.io.IOException;

import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletException;

import org.loom.servlet.LoomServletRequest;
import org.loom.servlet.LoomServletResponse;
import org.loom.servlet.Workflow;
import org.loom.util.Lookup;


/**
 * The registered loom filter.
 * @author Ignacio Coloma
 */
public class LoomFilter extends AbstractLoomFilter {
	
	/** the workflow instance. This class does all the work. */
	private static Workflow workflow;
	
	@Override
	protected void doFilter(LoomServletRequest request, LoomServletResponse response, FilterChain chain) throws IOException, ServletException {
		workflow.doFilter(request, response,  chain);
	}
	
	/**
	 * Create and configure the config instance
	 */
	@Override
	public void init(FilterConfig fc) throws ServletException {
		Lookup lookup = Lookup.getInstance();
		if (lookup == null) {
			throw new IllegalStateException("A Dependency Injection environment has not been initialized. Please check that your web.xml is properly configured");
		}
		workflow = lookup.byClass(Workflow.class);
		if (workflow == null) {
			throw new IllegalStateException("A configured Workflow instance could not be found.");
		}
		super.init(fc);
	}
	
}

