package org.loom.guice;

import java.util.Set;

import javax.inject.Inject;
import javax.inject.Singleton;

import org.loom.action.Action;
import org.loom.log.Log;
import org.loom.mapping.ActionMapping;
import org.loom.mapping.ActionMappingFactory;
import org.loom.mapping.DefaultActionMappingRepository;

import com.google.common.base.Function;
import com.google.common.collect.Collections2;

@Singleton
public class GuiceActionMappingRepository extends DefaultActionMappingRepository {

	@Inject 
	private ActionMappingFactory actionMappingFactory;
	
	private static Log log = Log .getLog(GuiceActionMappingRepository.class);
	
	public void setActionMappingFactory(ActionMappingFactory actionMappingFactory) {
		this.actionMappingFactory = actionMappingFactory;
	}

	@Inject
	public void setActionClasses(Set<Class<Action>> actionClasses) {
		if (log.isInfoEnabled()) {
			log.info("Registering action classes: ", Collections2.transform(actionClasses, new Function<Class<Action>, String>() {

				@Override
				public String apply(Class<Action> from) {
					return from.getName();
				}
				
			}));
		}
		for (Class<? extends Action> actionClass : actionClasses) {
			ActionMapping mapping = actionMappingFactory.create(actionClass);
			addActionMapping(mapping);
		}
		validateActionMappings();
	}
	
}
