package org.loom.guice;

import javax.annotation.security.DenyAll;
import javax.annotation.security.PermitAll;
import javax.annotation.security.RolesAllowed;

import org.loom.annotation.processor.AnnotationProcessor;
import org.loom.annotation.processor.CommonAnnotationsProcessor;
import org.loom.annotation.processor.SSLPolicyAnnotationProcessor;
import org.loom.interceptor.DefaultSecurityInterceptor;
import org.loom.interceptor.SecurityInterceptor;
import org.loom.security.JeeSecurityAdapter;
import org.loom.security.SecurityAdapter;

import com.google.inject.AbstractModule;
import com.google.inject.multibindings.Multibinder;

/**
 * Adds JSR250 annotations ( {@link RolesAllowed}, {@link PermitAll} and {@link DenyAll}) for security constraints
 * @author icoloma
 *
 */
public class SecurityModule extends AbstractModule {

	@Override
	protected void configure() {
		Multibinder<AnnotationProcessor> apBinder = Multibinder.newSetBinder(binder(), AnnotationProcessor.class);
	    apBinder.addBinding().to(CommonAnnotationsProcessor.class); 
	    apBinder.addBinding().to(SSLPolicyAnnotationProcessor.class); 
	    
	    // security
	    bind(SecurityInterceptor.class).to(DefaultSecurityInterceptor.class);
	    bind(SecurityAdapter.class).to(JeeSecurityAdapter.class);
	    
	    // SSL
	    // bind(DefaultSslPolicyInterceptor.class).to(DefaultSslPolicyInterceptor.class);
	}

}
