package org.loom.i18n;

import org.loom.log.Log;
import org.loom.servlet.LoomServletRequest;
import org.loom.servlet.LoomServletRequestImpl;

/**
 * Helper methods to manipulate {@link Messages} from service classes.
 * This class will not fail if there is no {@link LoomServletRequest} instance bound to this thread,
 * and instead the message will be logged.
 * @author icoloma
 *
 */
public class MessagesUtils {
	
	private static Log log = Log.getLog(MessagesUtils.class);
	
	public static Message error(String propertyPath, String messageKey) {
		return addMessage(MessageLevel.ERROR, propertyPath, messageKey);
	}
	
	public static Message error(String messageKey) {
		return addMessage(MessageLevel.ERROR, null, messageKey);
	}
	
	public static Message info(String messageKey) {
		return addMessage(MessageLevel.INFO, null, messageKey);
	}
	
	public static Message warn(String messageKey) {
		return addMessage(MessageLevel.WARN, null, messageKey);
	}
	
	private static Message addMessage(MessageLevel level, String propertyPath, String messageKey) {
		LoomServletRequest request = LoomServletRequestImpl.getThreadLocal();
		Message message = propertyPath == null? new Message(level, messageKey) :  new Message(level, messageKey, propertyPath);
		if (request == null || request.getParsedAction() == null) {
			log.info(level, messageKey);
			return message;
		}
		return request.getParsedAction().getMessages().addMessage(message);
	}
	
}
