package org.loom.interceptor;

import java.io.IOException;
import java.util.Iterator;

import javax.imageio.ImageIO;
import javax.imageio.ImageReader;
import javax.imageio.stream.ImageInputStream;

import org.loom.servlet.params.FileParameter;

/**
 * Default image validator instance
 * @author icoloma
 *
 */
public class DefaultImageValidationInterceptor extends AbstractImageValidationInterceptor {

    /**
     * The IOUtils.closeQuietly() method cannot be used with a ImageInputStream
     * @param iis
     */
	private void closeQuietly(ImageInputStream iis) {
		try {
			if (iis != null) {
				iis.close();
			}
		} catch (IOException e) {
			// ignored
		}
	}

	@Override
	protected ImageData getImageData(FileParameter fileParam) {
		ImageInputStream iis = null;
		try {
			iis = ImageIO.createImageInputStream(fileParam.getStream());
	        Iterator<ImageReader> iter = ImageIO.getImageReaders(iis);
	        boolean valid = iter.hasNext();
	        if (valid) {
	        	ImageReader reader = iter.next();
	        	reader.setInput(iis, true);
	        	ImageData data = new ImageData();
	        	data.setFormat(reader.getFormatName().toLowerCase());
	        	data.setWidth(reader.getWidth(0));
            	data.setHeight(reader.getHeight(0));
	        	return data;
	        }
	        return null;
		} catch (IOException e) {
			throw new RuntimeException(e);
		} finally {
			closeQuietly(iis);
		}
	}
	
}
