package org.loom.interceptor;

import static org.loom.url.UrlBuilder.INSECURE_SCHEME;
import static org.loom.url.UrlBuilder.SECURE_SCHEME;

import javax.annotation.PostConstruct;
import javax.inject.Inject;
import javax.inject.Singleton;

import org.loom.annotation.SSLPolicyType;
import org.loom.servlet.LoomServletRequest;
import org.springframework.security.web.PortMapper;
import org.springframework.security.web.PortMapperImpl;
import org.springframework.security.web.PortResolver;
import org.springframework.security.web.PortResolverImpl;

@Singleton
public class DefaultSSLRedirectDataFactory implements SSLRedirectDataFactory {
	
	/** will provide the http-to-https port mapping */
	@Inject
	private PortMapper portMapper;
	
	@Inject
	private PortResolver portResolver;
	
	@PostConstruct
	public void afterPropertiesSet() {
		if (portMapper == null) {
			portMapper = new PortMapperImpl();
		}
		if (portResolver == null) {
			portResolver = new PortResolverImpl();
			((PortResolverImpl)portResolver).setPortMapper(portMapper);
		}
	}

	@Override
	public SSLRedirectData getRedirectData(LoomServletRequest request, SSLPolicyType policy) {
		boolean secure = request.isSecure();
		Integer port = portResolver.getServerPort(request);
		
		if (secure && policy == SSLPolicyType.REQUIRES_INSECURE) {
			return new SSLRedirectData(INSECURE_SCHEME, null, portMapper.lookupHttpPort(port));
		} else if (!secure && policy == SSLPolicyType.REQUIRES_SECURE) {
			return new SSLRedirectData(SECURE_SCHEME, null, portMapper.lookupHttpsPort(port));
		}

		return null;
	}

	protected int lookupHttpPort(int httpsPort) {
		return portMapper.lookupHttpPort(httpsPort);
	}
	
	protected int lookupHttpsPort(int httpPort) {
		return portMapper.lookupHttpsPort(httpPort);
	}

	public void setPortMapper(PortMapper portMapper) {
		this.portMapper = portMapper;
	}

	public void setPortResolver(PortResolver portResolver) {
		this.portResolver = portResolver;
	}
}
