/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.interceptor;

import java.lang.reflect.Method;

import org.loom.mapping.ParsedAction;
import org.loom.resolution.Resolution;
import org.loom.util.ClassUtils;

/**
 * Interceptor that calls an action's listener method.
 * @author Ignacio Coloma
 */
public abstract class MethodInterceptor implements Interceptor {

	/** method to be called when the listener gets called */
	private Method method;
	
	/**
	 * @return a MethodInterceptor
	 */
	public static <T extends MethodInterceptor> T getInstance(Class<T> clazz, Method method) {
		T instance = ClassUtils.newInstance(clazz);
		instance.setMethod(method);
		return instance;
	}
	
	public void setMethod(Method method) {
		this.method = method;
	}
	
	protected Resolution invoke(ParsedAction action, Object... args) {
		return (Resolution) ClassUtils.invoke(action.getAction(), method, args);
	}
	
	@Override
	public String toString() {
		return getClass().getSimpleName() + "(" + method.getName() + ")";
	}
	
	public static class AfterCreate extends MethodInterceptor implements CreateInterceptor {
		
		public Resolution afterCreate(ParsedAction action) {
			return invoke(action);
		}
		
	}
	
	public static class BeforePopulate extends MethodInterceptor implements PopulateInterceptor {

		public Resolution beforePopulate(ParsedAction action) {
			return invoke(action);
		}

	}
	
	public static class BeforeValidate extends MethodInterceptor implements ValidateInterceptor {
		
		public Resolution beforeValidate(ParsedAction action) {
			return invoke(action);
		}
		
	}
	
	public static class BeforeExecute extends MethodInterceptor implements ExecuteInterceptor {
		
		public Resolution beforeExecute(ParsedAction action) {
			return invoke(action);
		}

	}
	
	public static class OnError extends MethodInterceptor implements ErrorListener {
		
		public Resolution onError(ParsedAction action) {
			return invoke(action);
		}
		
	}
	
	public static class DoFinally extends MethodInterceptor implements FinallyListener {

		public void doFinally(ParsedAction action, Exception e) {
			invoke(action, e);
		}
		
	}

}
