package org.loom.mapping;

import java.util.Map;
import java.util.Set;

import org.loom.binding.AnnotationContainer;
import org.loom.binding.PropertyBinder;
import org.loom.converter.Converter;
import org.loom.i18n.Messages;
import org.loom.i18n.MessagesRepository;
import org.loom.servlet.params.ServletRequestParameters;
import org.loom.validator.PropertyValidator;

/**
 * Argument received by an Event
 * 
 * @author icoloma
 */
public interface EventParameter extends AnnotationContainer {

	/**
	 * Return the list of created parameter values for this request
	 */
	public Object populate(Object container, ServletRequestParameters parameters, MessagesRepository repository, Messages messages);
	
	/**
	 * execute the validations configured for this event parameter
	 * 
	 * @param container
	 *            the event parameter value
	 * @param messages
	 *            where validation errors will be added
	 */
	public void validate(Object container, Messages messages);
	
	/**
	 * @param normalizedPropertyPathWithPrefix
	 *            the property path including the parameter name
	 */
	public PropertyBinder getPropertyBinder( String normalizedPropertyPathWithPrefix);
	
	/**
	 * @return true if this EventParameter handles the proposed property path
	 */
	public boolean matches(String propertyPath);
	
	public Object getPropertyAsObject(Object container, String propertyPath);

	public String getPropertyAsString(Object container, String propertyPath, MessagesRepository repository);
	
	/**
	 * Assigns value to some property nested inside this eventParameter
	 */
	public void setPropertyAsObject(Map<String, Object> parameterValues, String propertyPath, Object value);
	
	public Converter getConverter();
	
	public Class<?> getParameterType();

	public Set<PropertyValidator> getValidators(String propertyPath);

	public void addValidator(PropertyValidator validator);
}
