/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.persistence.file;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.InputStream;

import org.loom.exception.RuntimeFileNotFoundException;

/**
 * A file that is stored as a regular file in the filesystem. Does not use the database for anything.
 * 
 * @author icoloma
 */
public class SystemFileContents implements PersistentFileContents {

	/** file location */
	private File file;

	public SystemFileContents(File file) {
		this.file = file;
	}

	public InputStream getContentsAsStream() {
		try {
			return new FileInputStream(file);
		} catch (FileNotFoundException e) {
			throw new RuntimeFileNotFoundException(e);
		}
	}
	
	public Long getFileSize() {
		return file.exists()? null : file.length();
	}

}
