/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.resolution;

import java.io.IOException;

import javax.servlet.ServletException;

import org.loom.servlet.LoomServletRequest;
import org.loom.servlet.LoomServletResponse;

/**
 * Returns a string to the browser. No Content-Type will be set by default.
 * @author Ignacio Coloma
 */
public class StringResolution extends AbstractHttpResolution {

	/** the contents of the response */
	private StringBuilder contents;
	
	public StringResolution(int initialSize) {
		contents = new StringBuilder(initialSize);
	}
	
	public StringResolution() {
		this(1000);
	}
	
	public StringResolution(String contents) {
		this(contents.length());
		this.contents.append(contents);
	}

	public void resolve(LoomServletRequest request, LoomServletResponse response) throws IOException, ServletException {
		writeHeaders(request, response);
		response.getWriter().write(contents.toString());
	}
	
	public StringResolution append(CharSequence seq) {
		contents.append(seq);
		return this;
	}
	
	public StringResolution append(Object o) {
		contents.append(o);
		return this;
	}

	public StringBuilder getContents() {
		return contents;
	}

}
