package org.loom.resources;

import java.util.Collection;
import java.util.List;

import org.springframework.core.io.Resource;

import com.google.common.collect.Lists;

/**
 * The result of compiling a list of resources
 * @author icoloma
 *
 */
public class CompilationResult {

	/** the CSS/JS file contents */
	private List<CompiledWebResource> compiledResources;
	
	/** for CSS files, the refreenced resources (images used as url() attributes) */
	private List<CssResource> cssResources;
	
	/** the full list of Resources (after resolving dependencies) */
	private Collection<Resource> sources;

	public CompilationResult(WebResourceType type, int expectedSize, Collection<Resource> sources) {
		this.compiledResources = Lists.newArrayListWithExpectedSize(expectedSize);
		this.sources = sources;
		if (type == WebResourceType.CSS) {
			this.cssResources = Lists.newArrayList();
		}
	}
	
	public void addCssResource(CssResource resource) {
		cssResources.add(resource);
	}
	
	public void addCompiledWebResource(CompiledWebResource resource) {
		compiledResources.add(resource);
	}
	
	public List<CompiledWebResource> getCompiledResources() {
		return compiledResources;
	}

	public List<CssResource> getCssResources() {
		return cssResources;
	}

	public Collection<Resource> getSources() {
		return sources;
	}

}
