package org.loom.resources;

import java.io.IOException;
import java.io.InputStream;

import javax.servlet.jsp.JspException;

import org.loom.tags.core.AbstractWebResourceTag;

/**
 * The contents of a web resource (javascript or css file)
 * This class contains the result of the concatenation and
 * minification (obfuscation) of one or multiple files, and should be immutable.
 * 
 * @author icoloma
 */
public interface CompiledWebResource {

	public String getName();

	/**
	 * @return the calculated MD5 hash of this file
	 */
	public String getMd5();
	
	/**
	 * @return a stream to retrieve the resource data
	 */
	public InputStream getInputStream();
	
	/**
	 * @return a stream to retrieve a gzipped version of the resource data
	 */
	public InputStream getGzippedInputStream();

	/**
	 * @return the length of the resource data file
	 */
	public long getLength();
	
	/**
	 * @return the length of the gzipped resource data file
	 */
	public long getGzippedLength();
	
	/**
	 * render the tag for this compiled resource
	 */
	void render(AbstractWebResourceTag tag) throws JspException, IOException;
	
}
