package org.loom.resources;

import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import javax.inject.Inject;
import javax.inject.Singleton;

import org.springframework.core.io.Resource;

@Singleton
public class DefaultWebResourceBundleRepository implements WebResourceBundleRepository {
	
	/** the list of configured WebResourceBundle, the name is key */
	private Map<String, WebResourceBundle> bundles = new ConcurrentHashMap<String, WebResourceBundle>();
	
	@Inject
	private WebResourceBundleFactory webResourceBundleFactory;
	
	@Override
	public WebResourceBundle getWebResourceBundle(String name) {
		WebResourceBundle bundle = bundles.get(name);
		if (bundle == null) {
			bundle = webResourceBundleFactory.create(name);
			
			// store under the actual location and under the bundle name
			bundles.put(name, bundle);
			bundles.put(bundle.getName(), bundle);
		}
		return bundle;
	}
	
	@Override
	public WebResourceBundle addResourceBundle(String name, List<Resource> resources) {
		WebResourceBundle bundle = webResourceBundleFactory.create(name, resources);
		bundles.put(name, bundle);
		return bundle;
	}
	
	public void setBundles(Collection<WebResourceBundle> bundles) {
		for (WebResourceBundle bundle : bundles) {
			this.bundles.put(bundle.getName(), bundle);
		}
	}

	public void setWebResourceBundleFactory( WebResourceBundleFactory webResourceBundleFactory) {
		this.webResourceBundleFactory = webResourceBundleFactory;
	}

	
}
