package org.loom.resources;

import java.io.IOException;

import javax.servlet.jsp.JspException;

import org.loom.tags.core.AbstractWebResourceTag;
import org.loom.url.UrlBuilder;

public class PrecompiledWebResourceBundle implements WebResourceBundle {

	private String name;
	
	private String location;
	
	private WebResourceType type;

	public PrecompiledWebResourceBundle(String name, String location) {
		if (location == null) {
			throw new IllegalArgumentException("Cannot find bundle " + name + " in the precompiled index. This usually means that you have not precompiled before deploying.");
		}
		this.name = name;
		this.type = WebResourceType.identify(location);
		this.location = location;
	}

	@Override
	public void render(AbstractWebResourceTag tag) throws JspException, IOException {
		UrlBuilder url = tag.getRequest().createUrl(location);
		tag.setUrl(url);
		tag.printTag();
	}
	
	@Override
	public String getName() {
		return name;
	}

	@Override
	public WebResourceType getType() {
		return type;
	}

	@Override
	public CompiledWebResource getCompiledWebResource(String name) {
		throw new UnsupportedOperationException();
	}

	@Override
	public CssResource getCssResource(String name) {
		throw new UnsupportedOperationException("Only absolute-positioned CSS resources (start with '/') are available when using precompiled web resources");
	}

	public String getLocation() {
		return location;
	}

}
