package org.loom.resources;

import java.io.IOException;
import java.util.Map;
import java.util.concurrent.ConcurrentMap;

import javax.servlet.jsp.JspException;

import org.loom.tags.core.AbstractWebResourceTag;
import org.springframework.core.io.Resource;

import com.google.common.collect.Maps;

public class ReloadableWebResourceBundle extends DefaultWebResourceBundle {

	private ConcurrentMap<Resource, Long> resourcesTimestamps = Maps.newConcurrentMap();
	
	@Override
	public void render(AbstractWebResourceTag tag) throws JspException, IOException {
		for (Map.Entry<Resource, Long> entry : resourcesTimestamps.entrySet()) {
			Resource resource = entry.getKey();
			Long timestamp = entry.getValue();
			if (timestamp != -1) {
				if (timestamp != resource.getFile().lastModified()) {
					reload();
					break;
				}
			}
		}
		super.render(tag);
	}
	
	@Override
	protected synchronized CompilationResult compile() {
		CompilationResult compilationResult = super.compile();
		for (Resource resource : compilationResult.getSources()) {
			try {
				resourcesTimestamps.put(resource, resource.getFile().lastModified());
			} catch (IOException e) {
				resourcesTimestamps.put(resource, -1L);
			}
		}
		if (compilationResult.getCssResources() != null) {
			for (CssResource cssResource : compilationResult.getCssResources()) {
				Resource resource = cssResource.getResource();
				try {
					resourcesTimestamps.put(resource, resource.getFile().lastModified());
				} catch (IOException e) {
					resourcesTimestamps.put(resource, -1L);
				}
			}
		}
		return compilationResult;
	}
	
	private void reload() {
		this.compiledWebResources = null;
		this.cssResources = null;
	}
}
