package org.loom.servlet;

/**
 * The list of valid HTTP methods.
 * Note that the default method used by the browser can be overiden in a POST 
 * request by specifying a _method POST parameter.
 * For more information, see http://www.w3.org/Protocols/rfc2616/rfc2616-sec9.html  
 * Neither TRACE or CONNECT have been included.
 * @author icoloma
 *
 */
public enum HttpMethod {

	/**
	 * <a href="http://www.w3.org/Protocols/rfc2616/rfc2616-sec9.html#sec9.3">GET</a> is
	 * an idempotent action usually associated with data retrieval. 
	 */
	GET, 

	/**
	 * <a href="http://www.w3.org/Protocols/rfc2616/rfc2616-sec9.html#sec9.4">HEAD</a> is
	 * like GET, but should only return http headers with no body.
	 */
	HEAD, 
	
	/**
	 * <a href="http://www.w3.org/Protocols/rfc2616/rfc2616-sec9.html#sec9.5">POST</a> is
	 * a write operation usually associated with data insertions.
	 */
	POST, 
	
	/**
	 * <a href="http://www.w3.org/Protocols/rfc2616/rfc2616-sec9.html#sec9.6">PUT</a> is
	 * a write operation usually associated with data updates.
	 */
	PUT, 
	
	/**
	 * <a href="http://www.w3.org/Protocols/rfc2616/rfc2616-sec9.html#sec9.7">DELETE</a> is
	 * a data delete operation.
	 */
	DELETE, 
	
	/** 
	 * <a href="http://www.w3.org/Protocols/rfc2616/rfc2616-sec9.html#sec9.2">OPTIONS</a> is
	 * a header that offers a list of available HTTP methods. Its use is very rare. 
	 */
	OPTIONS;
	
}
