/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.servlet;

import java.io.IOException;

import javax.servlet.ServletResponse;
import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpServletResponseWrapper;


/**
 * Wrap the real response object.
 * @author icoloma
 */
public class LoomServletResponseImpl extends HttpServletResponseWrapper implements LoomServletResponse {
	
	/** the assigned status code, if any. Defaults to 200 (OK) */
	private int status = SC_OK;

    LoomServletResponseImpl(ServletResponse nested, LoomServletResponseData data) {
    	super((HttpServletResponse)nested);
    	setLocale(data.getLocale());
    	
    	// override the ISO-8859-1 specified by the spec
		setCharacterEncoding(data.getCharset());
    }

	public void deleteCookie(String name) {
		Cookie c = new Cookie(name, "deleted");
	    c.setMaxAge(0); 
	    c.setPath("/"); 
	    super.addCookie(c);
	}
	
	/**
	 * The addCookie behaviour has been changed. By default cookies are asociated to the webapp context,
	 * since it is the common case.
	 */
	@Override
	public void addCookie(Cookie cookie) {
		if (cookie.getPath() == null) {
			cookie.setPath("/");
		}
		super.addCookie(cookie);
	}
	
	@Override
	public Cookie addCookie(String name, String value) {
		return this.addCookie(name, value, -1);
	}
	
	@Override
	public Cookie addCookie(String name, String value,  int expiry) {
		Cookie cookie = new Cookie(name, value);
		cookie.setMaxAge(expiry);
		this.addCookie(cookie);
		return cookie;
	}

	@Override
	public void setStatus(int sc) {
		super.setStatus(sc);
		this.status = sc;
	}
	
	@Override
	public void sendError(int sc) throws IOException {
		super.sendError(sc);
		this.status = sc;
	}
	
	@Override
	public void sendError(int sc, String msg) throws IOException {
		super.sendError(sc, msg);
		this.status = sc;
	}
	
	public int getStatus() {
		return status;
	}
	
}
