/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.servlet.params;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;

import org.apache.commons.fileupload.FileItemStream;
import org.apache.commons.io.FilenameUtils;
import org.apache.commons.io.IOUtils;
import org.apache.commons.lang.builder.ToStringBuilder;
import org.loom.util.MimeUtils;

/**
 * Uploaded file managed in memory
 * @author Ignacio Coloma
 */
public class InMemoryFileParameter implements FileParameter {

	/** the filename without path */
	private String filename;
	
	/** the contentType */
	private String contentType;
	
	/** the size of the file, -1 if the browser did not provide it */
	private long size;

	/** the file contents */
	private byte[] contents;
	
	public InMemoryFileParameter(FileItemStream item) {
		this.filename = FilenameUtils.getName(item.getName());
		this.contentType = item.getContentType();
		if (this.contentType == null) {
			this.contentType = MimeUtils.guessMimeType(filename);
		}
		
		// read the file  contents
		InputStream is = null;
		try {
			is = item.openStream();
			this.contents = IOUtils.toByteArray(is);
		} catch (IOException e) {
			throw new RuntimeException(e);
		} finally {
			IOUtils.closeQuietly(is);
		}
		this.size = this.contents.length;
	}

	public byte[] getBytes() {
		return contents;
	}
	
	public InputStream getStream() throws IOException {
		return new ByteArrayInputStream(contents);
	}

	public String getContentType() {
		return contentType;
	}
	
	public void setContentType(String contentType) {
		this.contentType = contentType;
	}

	public long getFileSize() {
		return size;
	}

	public String getFilename() {
		return filename;
	}
	
	@Override
	public String toString() {
		return new ToStringBuilder(this)
			.append("filename", filename)
			.append("contentType", contentType)
			.append("size", size)
			.toString()
			;
	}
}
