package org.loom.servlet.scope;

import static org.loom.servlet.names.RequestAttributeNames.FLASH_NEXT;
import static org.loom.servlet.names.SessionAttributeNames.FLASH_PREFIX;

import java.util.concurrent.atomic.AtomicLong;

import javax.inject.Singleton;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

import org.loom.servlet.LoomServletRequest;
import org.loom.util.StringUtils;

@Singleton
public class SessionFlashContextFactory implements FlashContextFactory {

    /** generates the identifiers for flash contexts */
    private AtomicLong idGenerator = new AtomicLong();

	public FlashContext create(LoomServletRequest request) {
		FlashContext instance;
		HttpSession session = request.getSession(true);
		synchronized (session) {
			instance = (FlashContext) request.getAttribute(FLASH_NEXT);
			if (instance == null) {
				String id = StringUtils.numberToAlpha(idGenerator.getAndIncrement());
				instance = new FlashContext(id); 
				session.setAttribute(FLASH_PREFIX + id, instance);
			}
		}
		return instance;
	}

	/**
	 * Default implementation to retrieve a flash context using the session object
	 */
	public FlashContext retrieve(HttpServletRequest request, String flashID) {
		HttpSession session = request.getSession(false);
		FlashContext instance = null;
		if (session != null) {
			String flashAttributeId = FLASH_PREFIX + flashID;
			instance = (FlashContext) session.getAttribute(flashAttributeId);
			session.removeAttribute(flashAttributeId);
		}
		return instance;
	}

}
