package org.loom.spring;

import static org.loom.config.ClassNames.JAVAEE_5;

import java.util.Arrays;
import java.util.List;
import java.util.Set;

import org.codehaus.jackson.map.ObjectMapper;
import org.loom.action.Action;
import org.loom.annotation.processor.AnnotationProcessorRepository;
import org.loom.annotation.processor.BasicAnnotationProcessor;
import org.loom.annotation.processor.BrowserMessagesAnnotationProcessor;
import org.loom.annotation.processor.CacheAnnotationProcessor;
import org.loom.annotation.processor.ColumnAnnotationProcessor;
import org.loom.annotation.processor.CommonAnnotationsProcessor;
import org.loom.annotation.processor.DateValidationAnnotationProcessor;
import org.loom.annotation.processor.EagerPropertyAnnotationProcessor;
import org.loom.annotation.processor.EventAnnotationProcessor;
import org.loom.annotation.processor.FileValidationAnnotationProcessor;
import org.loom.annotation.processor.ImageValidationAnnotationProcessor;
import org.loom.annotation.processor.NumberValidationAnnotationProcessor;
import org.loom.annotation.processor.RequiredValidationAnnotationProcessor;
import org.loom.annotation.processor.RestAnnotationProcessor;
import org.loom.annotation.processor.RetrieveEntityAnnotationProcessor;
import org.loom.annotation.processor.SecuredAnnotationProcessor;
import org.loom.annotation.processor.StringValidationAnnotationProcessor;
import org.loom.binding.ItemMetadataFactory;
import org.loom.binding.NodeFactory;
import org.loom.binding.PropertyBinderFactory;
import org.loom.binding.PropertyWrapperFactory;
import org.loom.binding.factories.DefaultNodeFactory;
import org.loom.binding.factories.ImplementationFactoriesRepository;
import org.loom.config.ClassNames;
import org.loom.config.Config;
import org.loom.config.CookieLocaleResolver;
import org.loom.config.LocaleResolver;
import org.loom.converter.ConverterFactory;
import org.loom.converter.JsonConverterFactory;
import org.loom.exception.DefaultExceptionHandler;
import org.loom.exception.ExceptionHandler;
import org.loom.i18n.DefaultMessagesRepositoryFactory;
import org.loom.i18n.DefaultMissingMessageHandler;
import org.loom.i18n.MissingMessageHandler;
import org.loom.i18n.ReloadableMessagesRepositoryFactory;
import org.loom.interceptor.DefaultSecurityInterceptor;
import org.loom.log.Log;
import org.loom.mapping.ActionMappingFactory;
import org.loom.mapping.ActionMappingRepository;
import org.loom.mapping.DefaultActionMappingRepository;
import org.loom.mapping.ReloadableActionMappingRepository;
import org.loom.mapping.RestActionMappingFactory;
import org.loom.resolution.DefaultResolutionFactory;
import org.loom.resolution.ResolutionFactory;
import org.loom.resources.DefaultWebResourceBundle;
import org.loom.resources.DefaultWebResourceBundleFactory;
import org.loom.resources.DefaultWebResourceBundleRepository;
import org.loom.resources.InMemoryWebResourceCompiler;
import org.loom.resources.ReloadableWebResourceBundleFactory;
import org.loom.resources.ToDiskWebResourceCompiler;
import org.loom.resources.WebResourceBundleFactory;
import org.loom.resources.WebResourceBundleRepository;
import org.loom.resources.WebResourceCompiler;
import org.loom.resources.WebResourceType;
import org.loom.security.JeeSecurityAdapter;
import org.loom.security.SpringSecurityAdapter;
import org.loom.servlet.LoomServletRequestFactory;
import org.loom.servlet.Workflow;
import org.loom.servlet.params.DefaultFileParameterFactory;
import org.loom.servlet.params.FileParameterFactory;
import org.loom.servlet.scope.FlashContextFactory;
import org.loom.servlet.scope.SessionFlashContextFactory;
import org.loom.tags.ExpressionLanguageProxy;
import org.loom.tags.Jee5ExpressionLanguageProxy;
import org.loom.tags.LegacyExpressionLanguageProxy;
import org.loom.tags.decorator.TagDecoratorRepository;
import org.loom.util.ClassUtils;
import org.loom.util.LocaleUtils;
import org.loom.util.StringUtils;
import org.springframework.beans.factory.ListableBeanFactory;
import org.springframework.beans.factory.config.BeanDefinition;
import org.springframework.beans.factory.support.AbstractBeanDefinition;
import org.springframework.beans.factory.support.BeanDefinitionBuilder;
import org.springframework.beans.factory.support.BeanDefinitionRegistry;
import org.springframework.beans.factory.support.ManagedList;
import org.springframework.beans.factory.xml.AbstractBeanDefinitionParser;
import org.springframework.beans.factory.xml.ParserContext;
import org.springframework.context.annotation.ComponentScanBeanDefinitionParser;
import org.springframework.core.io.Resource;
import org.springframework.core.io.ResourceLoader;
import org.springframework.util.xml.DomUtils;
import org.w3c.dom.Element;

import com.google.common.collect.Sets;

/**
 * Parse the &lt;config> custom tag in spring config files.
 * 
 * @author icoloma
 * 
 */
public class ConfigBeanDefinitionParser extends AbstractBeanDefinitionParser {

	private static String NAMESPACE = "http://loom.extrema-sistemas.org/schema/loom";

	private BeanDefinitionRegistry beanRegistry;
	
	/** set of exceptions that must be rethrown instead of forwarded as HTTP error 500 */
	private Set<Class<? extends Throwable>> rethrowExceptions = null;

	private static Log log = Log.getLog(ConfigBeanDefinitionParser.class);
	
	private boolean development = true;

	@Override
	protected AbstractBeanDefinition parseInternal(Element element,
			ParserContext parserContext) {
		BeanDefinitionBuilder factory = BeanDefinitionBuilder.rootBeanDefinition(Config.class);
		beanRegistry = parserContext.getReaderContext().getRegistry();
		development = "true".equals(element.getAttribute("development"));

		addInternalClasses();
		addConverterFactory();
		addActionScanner(element, parserContext);
		addAnnotationProcessors();
		addExpressionLanguageProxy();
		addWebResourceBundles(DomUtils.getChildElementByTagName(element, "web-resources"), parserContext);
		addMessagesRepository(element);
		addLoomServletRequestFactory(element);
		addActionMappingRepository();
		addExceptionHandler();
		addIfMissing(TagDecoratorRepository.class, TagDecoratorRepository.class);
		addIfMissing(ResolutionFactory.class, DefaultResolutionFactory.class);
		addIfMissing(ObjectMapper.class, ObjectMapper.class);
		addIfMissing(Workflow.class, Workflow.class);
		add(SpringLookup.class);

		// config attributes
		/*
		String development = element.getAttribute("development");
		if (!StringUtils.isEmpty(development)) {
			factory.addPropertyValue("development", development);
		}*/

		/*
		 * BeanDefinitionBuilder configPostProcessor =
		 * BeanDefinitionBuilder.rootBeanDefinition(ConfigPostProcessor.class);
		 * beanRegistry.registerBeanDefinition("configPostProcessor",
		 * configPostProcessor.getBeanDefinition());
		 */

		return factory.getBeanDefinition();
	}
	
	private void addExceptionHandler() {
		ListableBeanFactory beanFactory = (ListableBeanFactory) beanRegistry;
		if (beanFactory.getBeanNamesForType(ExceptionHandler.class).length == 0) {
			BeanDefinitionBuilder factory = BeanDefinitionBuilder.rootBeanDefinition(DefaultExceptionHandler.class);
			if (rethrowExceptions != null) {
				factory.addPropertyValue("rethrowExceptions", rethrowExceptions);
			}
			beanRegistry.registerBeanDefinition(DefaultExceptionHandler.class.getSimpleName(), factory.getBeanDefinition());
		}
	}

	private void addExpressionLanguageProxy() {
		if (ClassUtils.isPresent(JAVAEE_5)) {
			addIfMissing(ExpressionLanguageProxy.class, Jee5ExpressionLanguageProxy.class); 
		} else {
			addIfMissing(ExpressionLanguageProxy.class, LegacyExpressionLanguageProxy.class); 
		}
	}


	protected void addLoomServletRequestFactory(Element element) {
		if (!beanRegistry.containsBeanDefinition("loomServletRequestFactory")) {
			BeanDefinitionBuilder factory = BeanDefinitionBuilder
					.rootBeanDefinition(LoomServletRequestFactory.class);
			beanRegistry.registerBeanDefinition("loomServletRequestFactory",
					factory.getBeanDefinition());
		}
		addIfMissing(MissingMessageHandler.class, DefaultMissingMessageHandler.class);
		addIfMissing(LocaleResolver.class, CookieLocaleResolver.class);
		addIfMissing(FlashContextFactory.class, SessionFlashContextFactory.class);
		addIfMissing(FileParameterFactory.class, DefaultFileParameterFactory.class);
	}


	private void addMessagesRepository(Element root) {
		if (!beanRegistry.containsBeanDefinition("messagesRepositoryFactory")) {
			
			BeanDefinitionBuilder mrfFactory = BeanDefinitionBuilder.rootBeanDefinition(DefaultMessagesRepositoryFactory.class);
			Element element = DomUtils.getChildElementByTagName(root, "messages");
			if (element != null) {
				String locations = element.getAttribute("locations");
				if (!StringUtils.isEmpty(locations)) {
					mrfFactory.addPropertyValue("locations", Arrays.asList(split(locations)));
				}
				String browserKeys = element.getAttribute("browserMessages");
				if (!StringUtils.isEmpty(browserKeys)) {
					mrfFactory.addPropertyValue("browserMessages", Sets.newHashSet(split(browserKeys)));
				}
				String defaultLocale = element.getAttribute("defaultLocale");
				if (!StringUtils.isEmpty(defaultLocale)) {
					mrfFactory.addPropertyValue("defaultLocale", LocaleUtils.toLocale(defaultLocale));
				}
			}

			if (development) {
				BeanDefinitionBuilder reloadableFactory = BeanDefinitionBuilder.rootBeanDefinition(ReloadableMessagesRepositoryFactory.class);
				reloadableFactory.addPropertyValue("delegate", mrfFactory.getBeanDefinition());
				beanRegistry.registerBeanDefinition("messagesRepositoryFactory", reloadableFactory.getBeanDefinition());
			} else {
				beanRegistry.registerBeanDefinition("messagesRepositoryFactory", mrfFactory.getBeanDefinition());
			}
			
		}
	}

	/**
	 * Add some annotation processors if the dependent classes are found in the
	 * classpath
	 */
	@SuppressWarnings("unchecked")
	protected void addAnnotationProcessors() {
		addIfMissing(AnnotationProcessorRepository.class, SpringAnnotationProcessorRepository.class);

	    add(EventAnnotationProcessor.class); 
	    add(EagerPropertyAnnotationProcessor.class); 
	    add(BrowserMessagesAnnotationProcessor.class); 
	    add(CacheAnnotationProcessor.class); 
	    add(RestAnnotationProcessor.class); 
	    add(StringValidationAnnotationProcessor.class); 
	    add(NumberValidationAnnotationProcessor.class); 
	    add(RequiredValidationAnnotationProcessor.class); 
	    add(DateValidationAnnotationProcessor.class); 
	    add(FileValidationAnnotationProcessor.class);
	    add(ImageValidationAnnotationProcessor.class);

		if (ClassUtils.isPresent("javax.ws.rs.Path")) {
			log.info("Adding JAX-RS annotations support.");
			add(RestAnnotationProcessor.class);
		}
		if (ClassUtils.isPresent("javax.persistence.Entity")) {
			log.info("Adding jpa annotations support.");
			add(BasicAnnotationProcessor.class);
			add(ColumnAnnotationProcessor.class);
			add(RetrieveEntityAnnotationProcessor.class);
		}
		
		boolean security = false;
		if (ClassUtils.isPresent(ClassNames.COMMONS_ANNOTATIONS)) {
			add(CommonAnnotationsProcessor.class);
			security = true;
		}
		if (ClassUtils.isPresent(ClassNames.SPRING_SECURITY)) {
			add(SecuredAnnotationProcessor.class);
			security = true;
			rethrowExceptions = (Set)Sets.newHashSet(ClassUtils.forName(ClassNames.SPRING_SECURITY_ACCESS_DENIED_EXCEPTION));
		}
		if (security) {
			BeanDefinitionBuilder factory = BeanDefinitionBuilder.rootBeanDefinition(DefaultSecurityInterceptor.class);
			factory.setScope("prototype");
			beanRegistry.registerBeanDefinition(DefaultSecurityInterceptor.class.getSimpleName(), factory.getBeanDefinition());
			add(ClassUtils.isPresent(ClassNames.SPRING_SECURITY) ? SpringSecurityAdapter.class : JeeSecurityAdapter.class);
		}
		
	}

	protected void addInternalClasses() {
		addIfMissing(PropertyWrapperFactory.class, PropertyWrapperFactory.class);
		addIfMissing(ImplementationFactoriesRepository.class, ImplementationFactoriesRepository.class);
		addIfMissing(ItemMetadataFactory.class, ItemMetadataFactory.class);
		addIfMissing(PropertyBinderFactory.class, PropertyBinderFactory.class);
		addIfMissing(NodeFactory.class, DefaultNodeFactory.class);
	}
	
	private void addConverterFactory() {
		addIfMissing(ConverterFactory.class, SpringConverterFactory.class);
		add(JsonConverterFactory.class);
	}

	protected void addIfMissing(Class<?> interfaceClass, Class<?> clazz) {
		ListableBeanFactory beanFactory = (ListableBeanFactory) beanRegistry;
		if (beanFactory.getBeanNamesForType(interfaceClass).length == 0) {
			BeanDefinitionBuilder factory = BeanDefinitionBuilder.rootBeanDefinition(clazz);
			beanRegistry.registerBeanDefinition(clazz.getSimpleName(), factory.getBeanDefinition());
		}
	}
	
	protected void add(Class<?> clazz) {
		BeanDefinitionBuilder factory = BeanDefinitionBuilder.rootBeanDefinition(clazz);
		beanRegistry.registerBeanDefinition(clazz.getSimpleName(), factory.getBeanDefinition());
	}

	/**
	 * Parse the action-scan component. The point is generating the following
	 * code:
	 * 
	 * <pre>
	 * &lt;context:component-scan base-package="org.loom.demo.action, org.loom.action.support" scope-resolver="org.loom.spring.PrototypeScopeResolver">
	 *    &lt;context:include-filter type="assignable" expression="org.loom.action.Action"/>
	 * &lt;/context:component-scan>
	 * </pre>
	 * 
	 * @param element
	 * @param parserContext
	 */
	protected void addActionScanner(Element element, ParserContext parserContext) {
		Element actionScan = DomUtils.getChildElementByTagName(element, "action-scan");
		actionScan.setAttribute("use-default-filters", "false");
		if (StringUtils.isEmpty(actionScan.getAttribute("scope-resolver"))) {
			actionScan.setAttribute("scope-resolver", PrototypeScopeResolver.class.getName());
		}
		Element includeFilter = element.getOwnerDocument().createElementNS( NAMESPACE, "include-filter");
		includeFilter.setAttribute("type", "assignable");
		includeFilter.setAttribute("expression", Action.class.getName());
		actionScan.appendChild(includeFilter);
		parseComponentScan(actionScan, parserContext);
		add(SpringActionScanner.class);
	}

	protected void parseComponentScan(Element element,
			ParserContext parserContext) {
		ComponentScanBeanDefinitionParser parser = new ComponentScanBeanDefinitionParser();
		parser.parse(element, parserContext);
	}

	private void addActionMappingRepository() {
		BeanDefinitionBuilder amfFactory = BeanDefinitionBuilder.rootBeanDefinition(RestActionMappingFactory.class);
		beanRegistry.registerBeanDefinition(ActionMappingFactory.class.getSimpleName(), amfFactory.getBeanDefinition());
		
		if (ClassUtils.isPresent(ClassNames.JAVAREBEL)) {
			log.info("JavaRebel detected in classpath, class reloading is enabled");
			BeanDefinitionBuilder amrFactory = BeanDefinitionBuilder.rootBeanDefinition(ReloadableActionMappingRepository.class);
			amrFactory.addPropertyValue("delegate", createDefaultActionMappingRepository());
			beanRegistry.registerBeanDefinition(ActionMappingRepository.class.getSimpleName(), amrFactory.getBeanDefinition());
		} else {
			log.info("JavaRebel is not in the classpath, class reloading is disabled");
			beanRegistry.registerBeanDefinition(ActionMappingRepository.class.getSimpleName(), createDefaultActionMappingRepository());
		}
	}
	
	private BeanDefinition createDefaultActionMappingRepository() {
		BeanDefinitionBuilder amfFactory = BeanDefinitionBuilder.rootBeanDefinition(DefaultActionMappingRepository.class);
		return amfFactory.getBeanDefinition();
		
	}

	protected void addWebResourceBundles(Element element, ParserContext parserContext) {
		BeanDefinitionBuilder webResourceBundleRepository = BeanDefinitionBuilder.rootBeanDefinition(DefaultWebResourceBundleRepository.class);
		BeanDefinitionBuilder webResourceBundleFactory = BeanDefinitionBuilder.rootBeanDefinition(development? ReloadableWebResourceBundleFactory.class : DefaultWebResourceBundleFactory.class);
		String storage = element.getAttribute("storage");
		addWebResourceCompiler(WebResourceCompiler.CSS, WebResourceType.CSS, storage);
		addWebResourceCompiler(WebResourceCompiler.JS, WebResourceType.JS, storage);

		List<BeanDefinition> bundles = new ManagedList<BeanDefinition>();
		if (element != null) {
			for (Element resource : DomUtils.getChildElementsByTagName(element, "web-resource")) {
				String name = resource.getAttribute("name");

				BeanDefinitionBuilder bundle = BeanDefinitionBuilder .rootBeanDefinition(DefaultWebResourceBundle.class);
				bundle.addPropertyValue("name", name);
				Set<Resource> resources = parseResourcesList( resource.getTextContent(), parserContext.getReaderContext() .getResourceLoader());
				if (resources.isEmpty()) {
					throw new IllegalArgumentException("ResourceBundle " + name + " is empty");
				}
				bundle.addPropertyValue("resources", resources);
				bundle.addPropertyReference( "compiler",
						WebResourceType.identify(resources.iterator().next()) == WebResourceType.CSS ? WebResourceCompiler.CSS : WebResourceCompiler.JS);
				bundles.add(bundle.getBeanDefinition());
			}
		}
		webResourceBundleRepository.addPropertyValue("bundles", bundles);
		beanRegistry.registerBeanDefinition(WebResourceBundleFactory.class.getSimpleName(), webResourceBundleFactory.getBeanDefinition());
		beanRegistry.registerBeanDefinition(WebResourceBundleRepository.class.getSimpleName(), webResourceBundleRepository.getBeanDefinition());
	}

	private BeanDefinition addWebResourceCompiler(String beanName, WebResourceType type, String storage) {
		BeanDefinitionBuilder webResourceCompiler = BeanDefinitionBuilder .rootBeanDefinition("disk".equals(storage) ? ToDiskWebResourceCompiler.class : InMemoryWebResourceCompiler.class);
		webResourceCompiler.addPropertyValue("development", development);
		webResourceCompiler.addPropertyValue("type", type);
		BeanDefinition beanDefinition = webResourceCompiler.getBeanDefinition();
		beanRegistry.registerBeanDefinition(beanName, beanDefinition);
		return beanDefinition;
	}

	private Set<Resource> parseResourcesList(String resourcesAsString,
			ResourceLoader resourceLoader) {
		String[] a = split(resourcesAsString);
		Set<Resource> resources = Sets.newLinkedHashSet();
		for (String resourcePath : a) {
			resources.add(resourceLoader.getResource(resourcePath));
		}
		return resources;
	}

	protected String[] split(String text) {
		return StringUtils.split(text, ", \n\t");
	}

	@Override
	protected boolean shouldGenerateIdAsFallback() {
		return true;
	}

}
