package org.loom.spring;

import javax.annotation.PostConstruct;
import javax.inject.Inject;
import javax.inject.Singleton;

import org.loom.action.Action;
import org.loom.mapping.ActionMapping;
import org.loom.mapping.ActionMappingFactory;
import org.loom.mapping.ActionMappingRepository;
import org.loom.util.ClassUtils;
import org.springframework.beans.factory.config.BeanDefinition;
import org.springframework.beans.factory.config.ConfigurableListableBeanFactory;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ConfigurableApplicationContext;

/**
 * Scan and register all {@link Action} instances detected by Spring
 * @author icoloma
 *
 */
@Singleton
public class SpringActionScanner {

	@Inject
	private ApplicationContext applicationContext;

	@Inject
	private ActionMappingFactory actionMappingFactory;
	
	@Inject
	private ActionMappingRepository actionMappingRepository;
	
	@PostConstruct
	public void scanAndRegisterActions() {
		ConfigurableListableBeanFactory beanFactory = ((ConfigurableApplicationContext)applicationContext).getBeanFactory();
		for (String name: beanFactory.getBeanNamesForType(Action.class)) {
			BeanDefinition bd = beanFactory.getBeanDefinition(name);
			Class<? extends Action> actionClass = (Class<? extends Action>) ClassUtils.forName(bd.getBeanClassName());
			ActionMapping mapping = actionMappingFactory.create(actionClass);
			actionMappingRepository.addActionMapping(mapping);
		}
		actionMappingRepository.validateActionMappings();
	}
	
	public void setApplicationContext(ApplicationContext applicationContext) {
		this.applicationContext = applicationContext;
	}

}
