/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.tags.core;

import java.io.IOException;

import javax.servlet.jsp.JspException;

import org.loom.config.ConfigProperties;
import org.loom.tags.AbstractFormInputTag;
import org.tldgen.annotations.Attribute;
import org.tldgen.annotations.BodyContent;
import org.tldgen.annotations.Tag;

/**
 * <a href="http://www.w3.org/TR/html4/interact/forms.html#file-select">HTML input file tag</a>.
 * Notice that file fields cannot be bound to java properties.
 * @author icoloma
 * @author rgrocha
 */
@Tag(
		dynamicAttributes=true,
		bodyContent=BodyContent.EMPTY,
		example="&lt;l:inputFile name=\"image\"/>"
	)
public class InputFileTag extends AbstractFormInputTag {
	
	private static String DEFAULT_CSS = ConfigProperties.getInstance().getDefaultCss(InputFileTag.class, "file");
	
	/** <a href="http://www.w3.org/TR/html4/interact/forms.html#adef-accept">HTML attribute</a> */
	@Attribute
	private String accept;

	public InputFileTag() {
		setCssClass(DEFAULT_CSS);
		setBound(false);
	}
	
	@Override
	public void doTagImpl() throws JspException, IOException {
		getForm().setMultipart(true);
		out.print("<input type=\"file\"");
		out.printAttribute("accept", accept);
		printParentAttributes();
		out.print("/>");
	}

	public void setAccept(String accept) {
		this.accept = accept;
	}

}
