package org.loom.tags.core;

import java.io.IOException;

import javax.servlet.jsp.JspException;

import org.loom.servlet.LoomServletRequest;
import org.loom.tags.AbstractTag;
import org.loom.url.ParameterContainer;
import org.loom.url.UrlBuilder;
import org.tldgen.annotations.Attribute;
import org.tldgen.annotations.BodyContent;
import org.tldgen.annotations.Tag;

/**
 * Renders a plain link. To render a full-blown url tag instead, see l:url
 * @author icoloma
 *
 */
@Tag(
		dynamicAttributes=true, 
		bodyContent=BodyContent.SCRIPTLESS
	)
public class LinkTag extends AbstractTag implements ParameterContainer {

	private UrlBuilder url;
	
	@Override
	public void setRequest(LoomServletRequest request) {
		super.setRequest(request);
		url = new UrlBuilder(request);
	}
	
	@Override
	public void doTagImpl() throws JspException, IOException {
		// process any nested parameters
		doBufferedBody();
		out.print(url.getURL());
	}

	public ParameterContainer addParameter(String name, Object value) {
		url.add(name, value);
		return this;
	}
	
	/**
	 * The action this link points to. Either this or href must be specified
	 */
	@Attribute
	public void setAction(String action) {
		this.url.setAction(action);
	}
	
	/**
	 * The event this link points to. If action is specified but event is not, the
	 * default event will be assumed. If the action does not define a default event,
	 * an exception will be thrown.
	 * Either action/event or href must be set.
	 */
	@Attribute
	public void setEvent(String event) {
		this.url.setEvent(event);
	}
	
	/**
	 * The mapping as <code>action.event</code>. This method is equivalent to 
	 * setting <code>action</code> and <code>event</code> separately.
	 */
	@Attribute
	public void setMapping(String mapping) {
		this.url.setMapping(mapping);
	}
	
	/**
	 * The hash of the url (the fragment after '#')
	 */
	@Attribute
	public void setHash(String hash) {
		this.url.setHash(hash);
	}
	
	/**
	 * The target url. Either this or the action / event pair must be set.
	 */
	@Attribute
	public void setHref(String href) {
		this.url.setRawUrl(href);
	}

}
