/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.tags.decorator;

import java.io.IOException;

import javax.servlet.jsp.JspException;

import org.loom.tags.WriterWrapper;
import org.loom.tags.core.InputRadioTag;


/**
 * Create a div around the radio tag
 * @author icoloma
 */
public class InputRadioDecorator extends AbstractFormInputDecorator<InputRadioTag> {

	@Override
	public boolean beforeDoTag(InputRadioTag tag) throws IOException {
		if (tag.shouldRenderLabel()) {
			// do not add the translated name attribute
			tag.setAddTranslatedName(false);
			WriterWrapper<?> out = tag.getWriter();
			out.print("<div");
			out.printAttribute("class", tag.getLabelCssClass());
			out.print("><span class=\"label\">");
			out.print(tag.getTranslatedLabel());
			out.print(":");
			out.print("</span>\n");
		}
		return true;
	}

	@Override
	public void afterDoTag(InputRadioTag tag) throws IOException, JspException {
		if (tag.shouldRenderLabel()) {
			tag.getWriter().print("</div>");
		}
		renderError(tag);
	}

}
