/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.tags.decorator;

import java.io.IOException;

import org.loom.i18n.MessagesRepository;
import org.loom.tags.WriterWrapper;
import org.loom.tags.core.InputHiddenTag;
import org.loom.util.HtmlSanitizer;


/**
 * Outputs the content of a hidden field
 * @author icoloma
 */
public class PrintHiddenFieldDecorator extends AbstractLoomDecorator<InputHiddenTag> {

	@Override
	public boolean beforeDoTag(InputHiddenTag tag) throws IOException {
		return true;
	}

	@Override
	public void afterDoTag(InputHiddenTag tag) throws IOException {
		MessagesRepository messagesRepository = tag.getRequest().getMessagesRepository();
		String name = tag.getName();
		
		WriterWrapper<?> out = tag.getWriter();
		out.print("<p><span class=\"label hidden-field\">");
		out.print(messagesRepository.guessString(name));
		out.print(":</span>");
		String value = HtmlSanitizer.sanitize(tag.getValue());
		out.print(value == null? messagesRepository.guessString(name + ".new") : value);
		out.print("</p>");
	}
	
}
