package org.loom.tags.messages;

import java.io.IOException;

import javax.servlet.jsp.JspException;
import javax.servlet.jsp.PageContext;

import org.loom.i18n.MessagesRepository;
import org.loom.servlet.Scope;
import org.loom.tags.AbstractTag;
import org.tldgen.annotations.Attribute;

public abstract class AbstractFormatTag extends AbstractTag {

	/** if not null, the value will be stored as an attribute instead of rendered in the page */
	@Attribute
	private String var;
	
	/** the scope of the variable, if set */
	private Scope scope = Scope.PAGE;
	
	protected MessagesRepository repository;
	
	@Override
	public void doTagImpl() throws JspException, IOException {
		// use this instead of threadLocal, in case this tag is being used by an error page
		repository = request.getMessagesRepository();
		String fv = format();
		if (var != null) {
			if (scope == Scope.PAGE) {
				PageContext pageContext = (PageContext)getJspContext();
				pageContext.setAttribute(var, fv, scope.getId());
			} else {
				request.setScopedAttribute(var, fv, scope);
			}
		} else {
			out.print(fv);
		}
	}
	
	protected abstract String format();
	
	public void setRepository(MessagesRepository repository) {
		this.repository = repository;
	}

	public void setVar(String var) {
		this.var = var;
	}

	/** 
	 * If var is not null, this is the scope where the value will be stored.
	 * Possible values are <code>page</code> (default), <code>request</code>,
	 * <code>session</code> and <code>application</code>.  
	 */
	@Attribute
	public void setScope(String scope) {
		this.scope = Scope.fromString(scope);
	}
}
