package org.loom.util.displaytag;

import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;

import org.apache.commons.lang.StringUtils;
import org.springframework.stereotype.Repository;

/**
 * Retrieve paged data in the format expected by DisplayTag.
 * This class could probably be refactored to use PagedListCriteria and PagedListData
 * @author icoloma
 *
 */
@Repository
public class DisplayTagDao {
	
    private EntityManager entityManager;

    @PersistenceContext
    public void setEntityManager(EntityManager entityManager) {
        this.entityManager = entityManager;
    }
	    
	/**
	 * Retrieve a paged query. The query can include a sorting order, which would then
	 * be appended to the sort indicated by the page object, if any.
	 *
	 * @param page The page to retrieve, including sorting order
	 * @param q The query string
	 * @param params The query params
	 */
	@SuppressWarnings("unchecked")
	public void query(PaginatedListImpl<?> page, String q, Object... params) {

		// add the order by clause
		String queryString = page.addOrderBy(q);

		// execute the query
		Query query = entityManager.createQuery(queryString);
		query.setFirstResult(page.getFirstRecordIndex());
		query.setMaxResults(page.getPageSize());
		setParameters(query, params);
		page.setList(query.getResultList());

		// obtains the total query results. delete possible "select" and "order by" clauses
        Query count = entityManager.createQuery("select count(*) from " + StringUtils.substringAfter(StringUtils.substringBefore(q, "order by"), "from"));
        setParameters(count, params);
        page.setTotal(((Number)count.getSingleResult()).intValue());
	}
	
	private void setParameters(Query query, Object... params) {
		if (params != null) {
			for (int i = 0; i < params.length; i++)
				query.setParameter(i + 1, params[i]);
		}
	}

}
