/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.validator;


import org.apache.commons.lang.builder.ToStringBuilder;
import org.loom.servlet.names.CssClassNames;
import org.loom.tags.FormInputTag;

/**
 * Parent of all the RequiredValidator classes
 * @param <T> The java class of fields validated by this validator
 * @author Ignacio Coloma
 */
public abstract class AbstractRequiredValidator<T> extends AbstractPropertyValidator {

	/** required simple property message key */
	public static final String REQUIRED_MESSAGE = "loom.validation.requiredFailed"; 
	
	/** required collection/array/map message key */
	public static final String REQUIRED_COLLECTION_MESSAGE = "loom.validation.requiredCollectionFailed"; 
	
	/** required boolean message key */
	public static final String REQUIRED_BOOLEAN_MESSAGE = "loom.validation.requiredBooleanFailed"; 
		
	/** default required message to use, if the user does not specify his own */
	private String defaultRequiredMessage;
	
	public AbstractRequiredValidator() {
		setMessagePrefix("required");
	}
	
    @SuppressWarnings("unchecked")
	public final void validate(ValidationRequest request, Object v) {
    	if (isEmpty((T) v)) { 
    		request.addErrorMessage(defaultRequiredMessage, v);
    	}
    }
    
    /** @return true if the value passed as parameter is empty */
    protected abstract boolean isEmpty(T value);
    
    @Override
    public void decorate(FormInputTag tag) {
    	super.decorate(tag);
		tag.addCssClass(CssClassNames.REQUIRED);
	}
 
    @Override
    public String toString() {
    	return new ToStringBuilder(this).append("propertyPath", getPropertyPath()).toString();
    }

	protected String getDefaultRequiredMessage() {
		return defaultRequiredMessage;
	}

	protected void setDefaultRequiredMessage(String defaultRequiredMessage) {
		this.defaultRequiredMessage = defaultRequiredMessage;
	}
	
}
