package org.loom.validator;

import org.loom.i18n.Message;
import org.loom.i18n.MessagesRepository;


/**
 * A Validator bound to a property.
 * Subclasses must be thread-safe.
 * 
 * @author icoloma
 *
 */
public interface PropertyValidator extends FormInputDecorator {
	
	/**
     * Validates one input value and adds to errors any validation error found 
     * @param value the value to validate
     */
    public void validate(ValidationRequest request, Object value);
    
    /**
     * The property path that will be evaluated.  Must be not-null and non-empty
     */
	public String getPropertyPath();

	/**
	 * If not null, this will be the error message key used instead of the one suggested by the 
	 * validator class.
	 * @return the message key to be used from the {@link MessagesRepository}, default null. 
	 */
	public String getOverridenMessageKey();
	
	/**
	 * @return true if this validation is applied to containers (lists, maps, arrays) instead of its contents.
	 * If false, the validation will be applied to each content, if true it will be applied to the container.
	 */
	public boolean validateCollectionContainer();
	
	/**
	 * @return true to bind error messages to the property name, false to use unbound {@link Message}
	 */
	public boolean isBindMessages();
}
